@@ -29,6 +29,7 @@
 #define _GNU_SOURCE
 #include "aconfig.h"
 #include <stdio.h>
+#include <stdint.h>
 #if HAVE_MALLOC_H
 #include <malloc.h>
 #endif
@@ -50,7 +51,6 @@
 #include <sys/uio.h>
 #include <sys/time.h>
 #include <sys/stat.h>
-#include <sys/types.h>
 #include <endian.h>
 #include "gettext.h"
 #include "formats.h"
@@ -101,6 +101,7 @@ static char *command;
 static snd_pcm_t *handle;
 static struct {
 	snd_pcm_format_t format;
+	snd_pcm_subformat_t subformat;
 	unsigned int channels;
 	unsigned int rate;
 } hwparams, rhwparams;
@@ -114,7 +115,7 @@ static int mmap_flag = 0;
 static int interleaved = 1;
 static int nonblock = 0;
 static volatile sig_atomic_t in_aborting = 0;
-static u_char *audiobuf = NULL;
+static uint8_t *audiobuf = NULL;
 static snd_pcm_uframes_t chunk_size = 0;
 static unsigned period_time = 0;
 static unsigned buffer_time = 0;
@@ -139,7 +140,7 @@ static snd_output_t *log;
 static long long max_file_size = 0;
 static int max_file_time = 0;
 static int use_strftime = 0;
-volatile static int recycle_capture_file = 0;
+static volatile int recycle_capture_file = 0;
 static long term_c_lflag = -1;
 static int dump_hw_params = 0;
 
@@ -216,6 +217,7 @@ _("Usage: %s [OPTION]... [FILE]...\n"
 "-t, --file-type TYPE    file type (voc, wav, raw or au)\n"
 "-c, --channels=#        channels\n"
 "-f, --format=FORMAT     sample format (case insensitive)\n"
+"    --subformat=SUBFORMAT  sample subformat (case insensitive)\n"
 "-r, --rate=#            sample rate\n"
 "-d, --duration=#        interrupt after # seconds\n"
 "-s, --samples=#         interrupt after # samples per channel\n"
@@ -412,7 +414,7 @@ static void signal_handler(int sig)
 }
 
 /* call on SIGUSR1 signal. */
-static void signal_handler_recycle (int sig)
+static void signal_handler_recycle(int sig ATTRIBUTE_UNUSED)
 {
 	/* flag the capture loop to start a new output file */
 	recycle_capture_file = 1;
@@ -434,6 +436,7 @@ enum {
 	OPT_USE_STRFTIME,
 	OPT_DUMP_HWPARAMS,
 	OPT_FATAL_ERRORS,
+	OPT_SUBFORMAT,
 };
 
 /*
@@ -491,6 +494,7 @@ int main(int argc, char *argv[])
 		{"file-type", 1, 0, 't'},
 		{"channels", 1, 0, 'c'},
 		{"format", 1, 0, 'f'},
+		{"subformat", 1, 0, OPT_SUBFORMAT},
 		{"rate", 1, 0, 'r'},
 		{"duration", 1, 0 ,'d'},
 		{"samples", 1, 0, 's'},
@@ -566,6 +570,7 @@ int main(int argc, char *argv[])
 
 	chunk_size = -1;
 	rhwparams.format = DEFAULT_FORMAT;
+	rhwparams.subformat = SND_PCM_SUBFORMAT_STD;
 	rhwparams.rate = DEFAULT_SPEED;
 	rhwparams.channels = 1;
 
@@ -635,6 +640,17 @@ int main(int argc, char *argv[])
 				}
 			}
 			break;
+		case OPT_SUBFORMAT:
+#if SND_LIB_VER(1, 2, 10) < SND_LIB_VERSION
+			rhwparams.subformat = snd_pcm_subformat_value(optarg);
+			if (rhwparams.subformat == SND_PCM_SUBFORMAT_UNKNOWN) {
+#else
+			if (strcasecmp(optarg, "std") != 0 && strcasecmp(optarg, "standard") != 0) {
+#endif
+				error(_("wrong extended subformat '%s'"), optarg);
+				prg_exit(EXIT_FAILURE);
+			}
+			break;
 		case 'r':
 			tmp = parse_long(optarg, &err);
 			if (err < 0) {
@@ -859,7 +875,7 @@ int main(int argc, char *argv[])
 	chunk_size = 1024;
 	hwparams = rhwparams;
 
-	audiobuf = (u_char *)malloc(1024);
+	audiobuf = (uint8_t *)malloc(1024);
 	if (audiobuf == NULL) {
 		error(_("not enough memory"));
 		return 1;
@@ -970,7 +986,7 @@ static int test_vocfile(void *buffer)
  * helper for test_wavefile
  */
 
-static size_t test_wavefile_read(int fd, u_char *buffer, size_t *size, size_t reqsize, int line)
+static size_t test_wavefile_read(int fd, uint8_t *buffer, size_t *size, size_t reqsize, int line)
 {
 	if (*size >= reqsize)
 		return *size;
@@ -995,17 +1011,17 @@ static size_t test_wavefile_read(int fd, u_char *buffer, size_t *size, size_t re
  *                            == 0 if not
  * Value returned is bytes to be discarded.
  */
-static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
+static ssize_t test_wavefile(int fd, uint8_t *_buffer, size_t size)
 {
 	WaveHeader *h = (WaveHeader *)_buffer;
-	u_char *buffer = NULL;
+	uint8_t *buffer = NULL;
 	size_t blimit = 0;
 	WaveFmtBody *f;
 	WaveChunkHeader *c;
-	u_int type, len;
+	uint32_t type, len;
 	unsigned short format, channels;
 	int big_endian, native_format;
-	u_char vbps = 0;
+	uint8_t vbps = 0;
 
 	if (size < sizeof(WaveHeader))
 		return -1;
@@ -1044,7 +1060,7 @@ static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
 
 	if (len < sizeof(WaveFmtBody)) {
 		error(_("unknown length of 'fmt ' chunk (read %u, should be %u at least)"),
-		      len, (u_int)sizeof(WaveFmtBody));
+		      len, (uint32_t)sizeof(WaveFmtBody));
 		prg_exit(EXIT_FAILURE);
 	}
 	check_wavefile_space(buffer, len, blimit);
@@ -1055,7 +1071,7 @@ static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
 		WaveFmtExtensibleBody *fe = (WaveFmtExtensibleBody*)buffer;
 		if (len < sizeof(WaveFmtExtensibleBody)) {
 			error(_("unknown length of extensible 'fmt ' chunk (read %u, should be %u at least)"),
-					len, (u_int)sizeof(WaveFmtExtensibleBody));
+					len, (unsigned int)sizeof(WaveFmtExtensibleBody));
 			prg_exit(EXIT_FAILURE);
 		}
 		if (memcmp(fe->guid_tag, WAV_GUID_TAG, 14) != 0) {
@@ -1167,7 +1183,7 @@ static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
 	size -= len;
 	
 	while (1) {
-		u_int type, len;
+		uint32_t type, len;
 
 		check_wavefile_space(buffer, sizeof(WaveChunkHeader), blimit);
 		test_wavefile_read(fd, buffer, &size, sizeof(WaveChunkHeader), __LINE__);
@@ -1372,6 +1388,11 @@ static void set_params(void)
 		show_available_sample_formats(params);
 		prg_exit(EXIT_FAILURE);
 	}
+	err = snd_pcm_hw_params_set_subformat(handle, params, hwparams.subformat);
+	if (err < 0) {
+		error(_("Sample subformat not available"));
+		prg_exit(EXIT_FAILURE);
+	}
 	err = snd_pcm_hw_params_set_channels(handle, params, hwparams.channels);
 	if (err < 0) {
 		error(_("Channels count non available"));
@@ -1473,6 +1494,17 @@ static void set_params(void)
 	err = snd_pcm_sw_params_set_stop_threshold(handle, swparams, stop_threshold);
 	assert(err >= 0);
 
+	if (verbose) {
+		err = snd_pcm_sw_params_set_tstamp_mode(handle, swparams, SND_PCM_TSTAMP_ENABLE);
+		assert(err >= 0);
+
+		if (monotonic)
+			err = snd_pcm_sw_params_set_tstamp_type(handle, swparams, SND_PCM_TSTAMP_TYPE_MONOTONIC);
+		else
+			err = snd_pcm_sw_params_set_tstamp_type(handle, swparams, SND_PCM_TSTAMP_TYPE_GETTIMEOFDAY);
+		assert(err >= 0);
+	}
+
 	if (snd_pcm_sw_params(handle, swparams) < 0) {
 		error(_("unable to install sw params:"));
 		snd_pcm_sw_params_dump(swparams, log);
@@ -1512,7 +1544,7 @@ static void set_params(void)
 			error(_("snd_pcm_mmap_begin problem: %s"), snd_strerror(err));
 			prg_exit(EXIT_FAILURE);
 		}
-		for (i = 0; i < hwparams.channels; i++)
+		for (i = 0; i < (int)hwparams.channels; i++)
 			fprintf(stderr, "mmap_area[%i] = %p,%u,%u (%u)\n", i, areas[i].addr, areas[i].first, areas[i].step, snd_pcm_format_physical_width(hwparams.format));
 		/* not required, but for sure */
 		snd_pcm_mmap_commit(handle, offset, 0);
@@ -1656,12 +1688,6 @@ static void xrun(void)
 		prg_exit(EXIT_FAILURE);
 	}
 	if (snd_pcm_status_get_state(status) == SND_PCM_STATE_XRUN) {
-		if (fatal_errors) {
-			error(_("fatal %s: %s"),
-					stream == SND_PCM_STREAM_PLAYBACK ? _("underrun") : _("overrun"),
-					snd_strerror(res));
-			prg_exit(EXIT_FAILURE);
-		}
 		if (monotonic) {
 #ifdef HAVE_CLOCK_GETTIME
 			struct timespec now, diff, tstamp;
@@ -1687,6 +1713,12 @@ static void xrun(void)
 			fprintf(stderr, _("Status:\n"));
 			snd_pcm_status_dump(status, log);
 		}
+		if (fatal_errors) {
+			error(_("fatal %s: %s"),
+					stream == SND_PCM_STREAM_PLAYBACK ? _("underrun") : _("overrun"),
+					snd_strerror(res));
+			prg_exit(EXIT_FAILURE);
+		}
 		if ((res = snd_pcm_prepare(handle))<0) {
 			error(_("xrun: prepare error: %s"), snd_strerror(res));
 			prg_exit(EXIT_FAILURE);
@@ -1809,7 +1841,7 @@ static void print_vu_meter(signed int *perc, signed int *maxperc)
 }
 
 /* peak handler */
-static void compute_max_peak(u_char *data, size_t samples)
+static void compute_max_peak(uint8_t *data, size_t samples)
 {
 	signed int val, max, perc[2], max_peak[2];
 	static int run = 0;
@@ -1895,7 +1927,7 @@ static void compute_max_peak(u_char *data, size_t samples)
 			else
 				val = be32toh(*valp);
 			val ^= mask;
-			if (val == 0x80000000U)
+			if ((unsigned int)val == 0x80000000U)
 				val = 0x7fffffff;
 			else
 				val = abs(val);
@@ -2054,9 +2086,9 @@ static void do_test_position(void)
 /*
  */
 #ifdef CONFIG_SUPPORT_CHMAP
-static u_char *remap_data(u_char *data, size_t count)
+static uint8_t *remap_data(uint8_t *data, size_t count)
 {
-	static u_char *tmp, *src, *dst;
+	static uint8_t *tmp, *src, *dst;
 	static size_t tmp_size;
 	size_t sample_bytes = bits_per_sample / 8;
 	size_t step = bits_per_frame / 8;
@@ -2090,9 +2122,9 @@ static u_char *remap_data(u_char *data, size_t count)
 	return tmp;
 }
 
-static u_char **remap_datav(u_char **data, size_t count)
+static uint8_t **remap_datav(uint8_t **data, size_t count ATTRIBUTE_UNUSED)
 {
-	static u_char **tmp;
+	static uint8_t **tmp;
 	unsigned int ch;
 
 	if (!hw_map)
@@ -2118,7 +2150,7 @@ static u_char **remap_datav(u_char **data, size_t count)
  *  write function
  */
 
-static ssize_t pcm_write(u_char *data, size_t count)
+static ssize_t pcm_write(uint8_t *data, size_t count)
 {
 	ssize_t r;
 	ssize_t result = 0;
@@ -2157,7 +2189,7 @@ static ssize_t pcm_write(u_char *data, size_t count)
 	return result;
 }
 
-static ssize_t pcm_writev(u_char **data, unsigned int channels, size_t count)
+static ssize_t pcm_writev(uint8_t **data, unsigned int channels, size_t count)
 {
 	ssize_t r;
 	size_t result = 0;
@@ -2210,7 +2242,7 @@ static ssize_t pcm_writev(u_char **data, unsigned int channels, size_t count)
  *  read function
  */
 
-static ssize_t pcm_read(u_char *data, size_t rcount)
+static ssize_t pcm_read(uint8_t *data, size_t rcount)
 {
 	ssize_t r;
 	size_t result = 0;
@@ -2252,7 +2284,7 @@ abort:
 	return result > rcount ? rcount : result;
 }
 
-static ssize_t pcm_readv(u_char **data, unsigned int channels, size_t rcount)
+static ssize_t pcm_readv(uint8_t **data, unsigned int channels, size_t rcount)
 {
 	ssize_t r;
 	size_t result = 0;
@@ -2304,7 +2336,7 @@ abort:
  *  ok, let's play a .voc file
  */
 
-static ssize_t voc_pcm_write(u_char *data, size_t count)
+static ssize_t voc_pcm_write(uint8_t *data, size_t count)
 {
 	ssize_t result = count, r;
 	size_t size;
@@ -2329,9 +2361,9 @@ static ssize_t voc_pcm_write(u_char *data, size_t count)
 static void voc_write_silence(unsigned x)
 {
 	unsigned l;
-	u_char *buf;
+	uint8_t *buf;
 
-	buf = (u_char *) malloc(chunk_bytes);
+	buf = (uint8_t *) malloc(chunk_bytes);
 	if (buf == NULL) {
 		error(_("can't allocate buffer for silence"));
 		return;		/* not fatal error */
@@ -2372,15 +2404,15 @@ static void voc_play(int fd, int ofs, char *name)
 	VocVoiceData *vd;
 	VocExtBlock *eb;
 	size_t nextblock, in_buffer;
-	u_char *data, *buf;
+	uint8_t *data, *buf;
 	char was_extended = 0, output = 0;
-	u_short *sp, repeat = 0;
+	uint16_t *sp, repeat = 0;
 	off_t filepos = 0;
 
 #define COUNT(x)	nextblock -= x; in_buffer -= x; data += x
 #define COUNT1(x)	in_buffer -= x; data += x
 
-	data = buf = (u_char *)malloc(64 * 1024);
+	data = buf = (uint8_t *)malloc(64 * 1024);
 	buffer_pos = 0;
 	if (data == NULL) {
 		error(_("malloc error"));
@@ -2404,6 +2436,7 @@ static void voc_play(int fd, int ofs, char *name)
 		}
 	}
 	hwparams.format = DEFAULT_FORMAT;
+	hwparams.subformat = SND_PCM_SUBFORMAT_STD;
 	hwparams.channels = 1;
 	hwparams.rate = DEFAULT_SPEED;
 	set_params();
@@ -2472,8 +2505,8 @@ static void voc_play(int fd, int ofs, char *name)
 #endif
 				break;
 			case 3:	/* a silence block, no data, only a count */
-				sp = (u_short *) data;
-				COUNT1(sizeof(u_short));
+				sp = (uint16_t *) data;
+				COUNT1(sizeof(uint16_t));
 				hwparams.rate = (int) (*data);
 				COUNT1(1);
 				hwparams.rate = 1000000 / (256 - hwparams.rate);
@@ -2488,8 +2521,8 @@ static void voc_play(int fd, int ofs, char *name)
 				voc_write_silence(*sp);
 				break;
 			case 4:	/* a marker for syncronisation, no effect */
-				sp = (u_short *) data;
-				COUNT1(sizeof(u_short));
+				sp = (uint16_t *) data;
+				COUNT1(sizeof(uint16_t));
 #if 0
 				d_printf("Marker %d\n", *sp);
 #endif
@@ -2503,8 +2536,8 @@ static void voc_play(int fd, int ofs, char *name)
 			case 6:	/* repeat marker, says repeatcount */
 				/* my specs don't say it: maybe this can be recursive, but
 				   I don't think somebody use it */
-				repeat = *(u_short *) data;
-				COUNT1(sizeof(u_short));
+				repeat = *(uint16_t *) data;
+				COUNT1(sizeof(uint16_t));
 #if 0
 				d_printf("Repeat loop %d times\n", repeat);
 #endif
@@ -2649,14 +2682,14 @@ static void begin_voc(int fd, size_t cnt)
 	}
 	bt.type = 1;
 	cnt += sizeof(VocVoiceData);	/* Channel_data block follows */
-	bt.datalen = (u_char) (cnt & 0xFF);
-	bt.datalen_m = (u_char) ((cnt & 0xFF00) >> 8);
-	bt.datalen_h = (u_char) ((cnt & 0xFF0000) >> 16);
+	bt.datalen = (uint8_t) (cnt & 0xFF);
+	bt.datalen_m = (uint8_t) ((cnt & 0xFF00) >> 8);
+	bt.datalen_h = (uint8_t) ((cnt & 0xFF0000) >> 16);
 	if (xwrite(fd, &bt, sizeof(VocBlockType)) != sizeof(VocBlockType)) {
 		error(_("write error"));
 		prg_exit(EXIT_FAILURE);
 	}
-	vd.tc = (u_char) (256 - (1000000 / hwparams.rate));
+	vd.tc = (uint8_t) (256 - (1000000 / hwparams.rate));
 	vd.pack = 0;
 	if (xwrite(fd, &vd, sizeof(VocVoiceData)) != sizeof(VocVoiceData)) {
 		error(_("write error"));
@@ -2671,8 +2704,8 @@ static void begin_wave(int fd, size_t cnt)
 	WaveFmtBody f;
 	WaveChunkHeader cf, cd;
 	int bits;
-	u_int tmp;
-	u_short tmp2;
+	uint32_t tmp;
+	uint16_t tmp2;
 
 	/* WAVE cannot handle greater than 32bit (signed?) int */
 	if (cnt == (size_t)-2)
@@ -2686,11 +2719,11 @@ static void begin_wave(int fd, size_t cnt)
 	case SND_PCM_FORMAT_S16_LE:
 		bits = 16;
 		break;
+	case SND_PCM_FORMAT_S24_LE: /* S24_LE is 24 bits stored in 32 bit width with 8 bit padding */
 	case SND_PCM_FORMAT_S32_LE:
-        case SND_PCM_FORMAT_FLOAT_LE:
+	case SND_PCM_FORMAT_FLOAT_LE:
 		bits = 32;
 		break;
-	case SND_PCM_FORMAT_S24_LE:
 	case SND_PCM_FORMAT_S24_3LE:
 		bits = 24;
 		break;
@@ -2715,11 +2748,11 @@ static void begin_wave(int fd, size_t cnt)
 #if 0
 	tmp2 = (samplesize == 8) ? 1 : 2;
 	f.byte_p_spl = LE_SHORT(tmp2);
-	tmp = dsp_speed * hwparams.channels * (u_int) tmp2;
+	tmp = dsp_speed * hwparams.channels * (uint32_t) tmp2;
 #else
 	tmp2 = hwparams.channels * snd_pcm_format_physical_width(hwparams.format) / 8;
 	f.byte_p_spl = LE_SHORT(tmp2);
-	tmp = (u_int) tmp2 * hwparams.rate;
+	tmp = (uint32_t) tmp2 * hwparams.rate;
 #endif
 	f.byte_p_sec = LE_INT(tmp);
 	f.bit_p_spl = LE_SHORT(bits);
@@ -2786,9 +2819,9 @@ static void end_voc(int fd)
 	cnt += sizeof(VocVoiceData);	/* Channel_data block follows */
 	if (cnt > 0x00ffffff)
 		cnt = 0x00ffffff;
-	bt.datalen = (u_char) (cnt & 0xFF);
-	bt.datalen_m = (u_char) ((cnt & 0xFF00) >> 8);
-	bt.datalen_h = (u_char) ((cnt & 0xFF0000) >> 16);
+	bt.datalen = (uint8_t) (cnt & 0xFF);
+	bt.datalen_m = (uint8_t) ((cnt & 0xFF00) >> 8);
+	bt.datalen_h = (uint8_t) ((cnt & 0xFF0000) >> 16);
 	if (lseek(fd, length_seek, SEEK_SET) == length_seek)
 		xwrite(fd, &bt, sizeof(VocBlockType));
 }
@@ -2798,7 +2831,7 @@ static void end_wave(int fd)
 	WaveChunkHeader cd;
 	off_t length_seek;
 	off_t filelen;
-	u_int rifflen;
+	uint32_t rifflen;
 	
 	length_seek = sizeof(WaveHeader) +
 		      sizeof(WaveChunkHeader) +
@@ -2869,7 +2902,7 @@ static void playback_go(int fd, size_t loaded, off_t count, int rtype, char *nam
 	while (written < count && !in_aborting) {
 		do {
 			c = count - written;
-			if (c > chunk_bytes)
+			if (c > (off_t)chunk_bytes)
 				c = chunk_bytes;
 
 			/* c < l, there is more data loaded
@@ -3300,7 +3333,7 @@ static void capture(char *orig_name)
 			if (read != f)
 				in_aborting = 1;
 			save = read * bits_per_frame / 8;
-			if (xwrite(fd, audiobuf, save) != save) {
+			if (xwrite(fd, audiobuf, save) != (ssize_t)save) {
 				perror(name);
 				in_aborting = 1;
 				break;
@@ -3339,7 +3372,7 @@ static void playbackv_go(int* fds, unsigned int channels, size_t loaded, off_t c
 	size_t vsize;
 
 	unsigned int channel;
-	u_char *bufs[channels];
+	uint8_t *bufs[channels];
 
 	header(rtype, names[0]);
 	set_params();
@@ -3393,7 +3426,7 @@ static void capturev_go(int* fds, unsigned int channels, off_t count, int rtype,
 	ssize_t r;
 	unsigned int channel;
 	size_t vsize;
-	u_char *bufs[channels];
+	uint8_t *bufs[channels];
 
 	header(rtype, names[0]);
 	set_params();
@@ -3436,14 +3469,14 @@ static void playbackv(char **names, unsigned int count)
 
 	if (count == 1 && channels > 1) {
 		size_t len = strlen(names[0]);
-		char format[1024];
-		memcpy(format, names[0], len);
-		strcpy(format + len, ".%d");
-		len += 4;
+		char buf[len + 1];
+		strcpy(buf, names[0]);
+		/* 1 for "." + 3 for channel (<= 256) + 1 for null terminator */
+		len += 5;
 		names = malloc(sizeof(*names) * channels);
 		for (channel = 0; channel < channels; ++channel) {
 			names[channel] = malloc(len);
-			sprintf(names[channel], format, channel);
+			snprintf(names[channel], len, "%s.%d", buf, channel);
 		}
 		alloced = 1;
 	} else if (count != channels) {
@@ -3489,14 +3522,14 @@ static void capturev(char **names, unsigned int count)
 
 	if (count == 1) {
 		size_t len = strlen(names[0]);
-		char format[1024];
-		memcpy(format, names[0], len);
-		strcpy(format + len, ".%d");
-		len += 4;
+		char buf[len + 1];
+		strcpy(buf, names[0]);
+		/* 1 for "." + 3 for channel (<= 256) + 1 for null terminator */
+		len += 5;
 		names = malloc(sizeof(*names) * channels);
 		for (channel = 0; channel < channels; ++channel) {
 			names[channel] = malloc(len);
-			sprintf(names[channel], format, channel);
+			snprintf(names[channel], len, "%s.%d", buf, channel);
 		}
 		alloced = 1;
 	} else if (count != channels) {
